<?php
namespace Ep\Wp\Plugin;

use MO;
use Psr\Cache\CacheItemPoolInterface;

/**
 * Kešira WordPress prijevode.
 *
 * @package Ep\Webshop\Plugin
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-wordpress
 */
class TranslationCache
{
    const CACHE_NAMESPACE = 'l10n';
    const CACHE_NAME      = 'cache.l10n';

    /** @var CacheItemPoolInterface */
    private $cache;

    /**
     * TranslationCache constructor.
     *
     * @param CacheItemPoolInterface $cache
     */
    public function __construct(CacheItemPoolInterface $cache)
    {
        $this->cache = $cache;

        add_filter('override_load_textdomain', [$this, 'override_load_textdomain'], PHP_INT_MAX, 3);
    }

    /**
     * Umjesto da prijevode parsamo iz .mo fajlova, čitamo ih iz keša.
     *
     * @param bool $overriden
     * @param string $domain
     * @param string $mofile
     *
     * @return bool
     */
    public function override_load_textdomain(bool $overriden, string $domain, string $mofile): bool
    {
        if (false === $overriden) {
            global $l10n;

            /**
             * Fires before the MO translation file is loaded.
             *
             * @since 2.9.0
             *
             * @param string $domain Text domain. Unique identifier for retrieving translated strings.
             * @param string $mofile Path to the .mo file.
             */
            do_action('load_textdomain', $domain, $mofile);

            /**
             * Filters MO file path for loading translations for a specific text domain.
             *
             * @since 2.9.0
             *
             * @param string $mofile Path to the MO file.
             * @param string $domain Text domain. Unique identifier for retrieving translated strings.
             */
            $mofile = apply_filters('load_textdomain_mofile', $mofile, $domain);

            if (!is_readable($mofile)) {
                return false;
            }

            $cacheKey = sprintf('%s.%s', $domain, md5($mofile));
            $cacheItem = $this->cache->getItem($cacheKey);

            if ($cacheItem->isHit()) {
                $data = unserialize($cacheItem->get());

                if (filemtime($mofile) > $data['mtime']) {
                    $this->cache->deleteItem($cacheItem->getKey());
                    $cacheItem = $this->cache->getItem($cacheKey);
                }
            }

            if (!$cacheItem->isHit()) {
                $mo = new MO();

                if ($mo->import_from_file($mofile)) {
                    $cacheItem->set(serialize([
                        'mtime'   => filemtime($mofile),
                        'headers' => $mo->headers,
                        'entries' => $mo->entries,
                    ]));

                    $this->cache->save($cacheItem);
                } else {
                    return false;
                }
            }

            $mo = new MO();
            $data = unserialize($cacheItem->get());

            $mo->headers = $data['headers'];
            $mo->entries = $data['entries'];

            $l10n[$domain] = $mo;
        }

        return true;
    }
}